@@ -164,7 +164,7 @@ public:
     };
 
     /**
-        Date calculations often depend on the country and wxDateTime allows to set
+        Date calculations often depend on the country and wxDateTime allows setting
         the country whose conventions should be used using SetCountry(). It takes
         one of the following values as parameter.
     */
@@ -263,7 +263,7 @@ public:
 
         This struct is analogous to standard C <code>struct tm</code> and uses
         the same, not always immediately obvious, conventions for its members:
-        notably its mon and mday fields count from 0 while yday counts from 1.
+        notably its mon and yday fields count from 0 while mday counts from 1.
      */
     struct Tm
     {
@@ -738,7 +738,7 @@ public:
     /**
        Returns the difference between this object and @a dt as a wxDateSpan.
 
-       This method allows to find the number of entire years, months, weeks and
+       This method allows finding the number of entire years, months, weeks and
        days between @a dt and this date.
 
        @since 2.9.5
@@ -1200,10 +1200,17 @@ public:
     //@{
 
     /**
-        Transform the date from the given time zone to the local one. If
-        @a noDST is @true, no DST adjustments will be made.
+        Transform the date from the given time zone to the local one.
 
-        @return The date in the local time zone.
+        If @a noDST is @true, no DST adjustments will be made.
+
+        Notice using wxDateTime::Local for @a tz parameter doesn't really make
+        sense and may result in unexpected results as it will return a
+        different object when DST is in use and @a noDST has its default value
+        of @false.
+
+        @return The date adjusted by the different between the given and the
+        local time zones.
     */
     wxDateTime FromTimezone(const TimeZone& tz, bool noDST = false) const;
 
@@ -1221,7 +1228,9 @@ public:
 
     /**
         Modifies the object in place to represent the date in another time
-        zone. If @a noDST is @true, no DST adjustments will be made.
+        zone.
+
+        If @a noDST is @true, no DST adjustments will be made.
     */
     wxDateTime& MakeTimezone(const TimeZone& tz, bool noDST = false);
 
@@ -1231,10 +1240,16 @@ public:
     wxDateTime& MakeUTC(bool noDST = false);
 
     /**
-        Transform the date to the given time zone. If @a noDST is @true, no DST
-        adjustments will be made.
+        Transform the date to the given time zone.
 
-        @return The date in the new time zone.
+        If @a noDST is @true, no DST adjustments will be made.
+
+        Notice that, as with FromTimezone(), using wxDateTime::Local as @a tz
+        doesn't really make sense and may return a different object when DST is
+        in effect and @a noDST is @false.
+
+        @return The date adjusted by the different between the local and the
+        given time zones.
     */
     wxDateTime ToTimezone(const TimeZone& tz, bool noDST = false) const;
 
