@@ -11,7 +11,7 @@
     the generic properties of the font without hardcoding in the sources a specific
     face name.
 
-    wxFontFamily thus allows to group the font face names of fonts with similar
+    wxFontFamily thus allows grouping the font face names of fonts with similar
     properties. Most wxWidgets ports use lists of fonts for each font family
     inspired by the data taken from http://www.codestyle.org/css/font-family.
 */
@@ -273,7 +273,7 @@ enum wxFontEncoding
     @class wxFontInfo
 
     This class is a helper used for wxFont creation using named parameter
-    idiom: it allows to specify various wxFont attributes using the chained
+    idiom: it allows specifying various wxFont attributes using the chained
     calls to its clearly named methods instead of passing them in the fixed
     order to wxFont constructors.
 
@@ -466,7 +466,7 @@ public:
 
         @since 2.9.5
      */
-    wxFont(const wxFontInfo& font);
+    wxFont(const wxFontInfo& fontInfo);
 
     /**
         Creates a font object with the specified attributes and size in points.
@@ -1036,7 +1036,7 @@ public:
     /**
         Sets the font size using a predefined symbolic size name.
 
-        This function allows to change font size to be (very) large or small
+        This function allows changing font size to be (very) large or small
         compared to the standard font size.
 
         @see SetSymbolicSizeRelativeTo().
