@@ -4,9 +4,8 @@
 // Author:      Julian Smart
 // Modified by: Dimitri Schoolwerth
 // Created:     26/10/98
-// RCS-ID:      $Id: notebook.cpp 63098 2010-01-08 04:33:21Z PC $
 // Copyright:   (c) 1998-2002 wxWidgets team
-// License:     wxWindows license
+// Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 // For compilers that support precompilation, includes "wx.h".
@@ -26,14 +25,21 @@
 #include "wx/utils.h"
 #include "notebook.h"
 
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
+//-----------------------------------------------------------------------------
+// MyApp
+//-----------------------------------------------------------------------------
+
 IMPLEMENT_APP(MyApp)
 
 bool MyApp::OnInit()
 {
+    if ( !wxApp::OnInit() )
+        return false;
+
 #if wxUSE_HELP
     wxHelpProvider::Set( new wxSimpleHelpProvider );
 #endif
@@ -54,6 +60,11 @@ bool MyApp::OnInit()
     return true;
 }
 
+
+//-----------------------------------------------------------------------------
+// Creation functions
+//-----------------------------------------------------------------------------
+
 wxPanel *CreateUserCreatedPage(wxBookCtrlBase *parent)
 {
     wxPanel *panel = new wxPanel(parent);
@@ -63,7 +74,7 @@ wxPanel *CreateUserCreatedPage(wxBookCtrlBase *parent)
 #endif
 
     (void) new wxButton( panel, wxID_ANY, wxT("Button"),
-        wxPoint(10, 10), wxDefaultSize );
+                         wxPoint(10, 10), wxDefaultSize );
 
     return panel;
 }
@@ -76,13 +87,15 @@ wxPanel *CreateRadioButtonsPage(wxBookCtrlBase *parent)
     panel->SetHelpText( wxT( "Panel with some Radio Buttons" ) );
 #endif
 
-    wxString animals[] = { wxT("Fox"), wxT("Hare"), wxT("Rabbit"),
+    wxString animals[] =
+      { wxT("Fox"), wxT("Hare"), wxT("Rabbit"),
         wxT("Sabre-toothed tiger"), wxT("T Rex") };
 
     wxRadioBox *radiobox1 = new wxRadioBox(panel, wxID_ANY, wxT("Choose one"),
         wxDefaultPosition, wxDefaultSize, 5, animals, 2, wxRA_SPECIFY_ROWS);
 
-    wxString computers[] = { wxT("Amiga"), wxT("Commodore 64"), wxT("PET"),
+    wxString computers[] =
+      { wxT("Amiga"), wxT("Commodore 64"), wxT("PET"),
         wxT("Another") };
 
     wxRadioBox *radiobox2 = new wxRadioBox(panel, wxID_ANY,
@@ -106,7 +119,8 @@ wxPanel *CreateVetoPage(wxBookCtrlBase *parent)
 #endif
 
     (void) new wxStaticText( panel, wxID_ANY,
-        wxT("This page intentionally left blank"), wxPoint(10, 10) );
+                             wxT("This page intentionally left blank"),
+                             wxPoint(10, 10) );
 
     return panel;
 }
@@ -128,7 +142,6 @@ wxPanel *CreateBigButtonPage(wxBookCtrlBase *parent)
     return panel;
 }
 
-
 wxPanel *CreateInsertPage(wxBookCtrlBase *parent)
 {
     wxPanel *panel = new wxPanel(parent);
@@ -139,7 +152,8 @@ wxPanel *CreateInsertPage(wxBookCtrlBase *parent)
 
     panel->SetBackgroundColour( wxColour( wxT("MAROON") ) );
     (void) new wxStaticText( panel, wxID_ANY,
-        wxT("This page has been inserted, not added."), wxPoint(10, 10) );
+                             wxT("This page has been inserted, not added."),
+                             wxPoint(10, 10) );
 
     return panel;
 }
@@ -197,11 +211,76 @@ wxPanel *CreatePage(wxBookCtrlBase *parent, const wxString&pageName)
     if ( pageName == MAXIMIZED_BUTTON_PAGE_NAME )
         return CreateBigButtonPage(parent);
 
-    wxFAIL_MSG( _T("unknown page name") );
+    wxFAIL_MSG( wxT("unknown page name") );
 
     return NULL;
 }
 
+
+//-----------------------------------------------------------------------------
+// MyFrame
+//-----------------------------------------------------------------------------
+
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
+    // File menu
+    EVT_MENU_RANGE(ID_BOOK_NOTEBOOK, ID_BOOK_MAX, MyFrame::OnType)
+    EVT_MENU_RANGE(ID_ORIENT_DEFAULT, ID_ORIENT_MAX, MyFrame::OnOrient)
+    EVT_MENU(ID_SHOW_IMAGES, MyFrame::OnShowImages)
+    EVT_MENU_RANGE(ID_FIXEDWIDTH, ID_HORZ_LAYOUT, MyFrame::OnStyle)
+    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
+
+    // Operations menu
+    EVT_MENU(ID_ADD_PAGE, MyFrame::OnAddPage)
+    EVT_MENU(ID_ADD_PAGE_NO_SELECT, MyFrame::OnAddPageNoSelect)
+    EVT_MENU(ID_INSERT_PAGE, MyFrame::OnInsertPage)
+    EVT_MENU(ID_DELETE_CUR_PAGE, MyFrame::OnDeleteCurPage)
+    EVT_MENU(ID_DELETE_LAST_PAGE, MyFrame::OnDeleteLastPage)
+    EVT_MENU(ID_NEXT_PAGE, MyFrame::OnNextPage)
+    EVT_MENU(ID_CHANGE_SELECTION, MyFrame::OnChangeSelection)
+    EVT_MENU(ID_SET_SELECTION, MyFrame::OnSetSelection)
+    EVT_MENU(ID_GET_PAGE_SIZE, MyFrame::OnGetPageSize)
+    EVT_MENU(ID_SET_PAGE_SIZE, MyFrame::OnSetPageSize)
+
+#if wxUSE_HELP
+    EVT_MENU(ID_CONTEXT_HELP, MyFrame::OnContextHelp)
+#endif // wxUSE_HELP
+    EVT_MENU(ID_HITTEST, MyFrame::OnHitTest)
+
+    // Book controls
+#if wxUSE_NOTEBOOK
+    EVT_NOTEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnNotebook)
+    EVT_NOTEBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnNotebook)
+#endif
+#if wxUSE_LISTBOOK
+    EVT_LISTBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnListbook)
+    EVT_LISTBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnListbook)
+#endif
+#if wxUSE_CHOICEBOOK
+    EVT_CHOICEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnChoicebook)
+    EVT_CHOICEBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnChoicebook)
+#endif
+#if wxUSE_TREEBOOK
+    EVT_TREEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnTreebook)
+    EVT_TREEBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnTreebook)
+
+    EVT_MENU(ID_ADD_SUB_PAGE, MyFrame::OnAddSubPage)
+    EVT_MENU(ID_ADD_PAGE_BEFORE, MyFrame::OnAddPageBefore)
+    EVT_UPDATE_UI_RANGE(ID_ADD_PAGE_BEFORE, ID_ADD_SUB_PAGE,
+                            MyFrame::OnUpdateTreeMenu)
+#endif
+#if wxUSE_TOOLBOOK
+    EVT_TOOLBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnToolbook)
+    EVT_TOOLBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnToolbook)
+#endif
+#if wxUSE_AUI
+    EVT_AUINOTEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnAuiNotebook)
+    EVT_AUINOTEBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnAuiNotebook)
+#endif
+
+    // Update title in idle time
+    EVT_IDLE(MyFrame::OnIdle)
+wxEND_EVENT_TABLE()
+
 MyFrame::MyFrame()
     : wxFrame(NULL, wxID_ANY, wxString(wxT("wxWidgets book controls sample")))
 {
@@ -217,13 +296,21 @@ MyFrame::MyFrame()
     m_type = Type_Listbook;
 #elif wxUSE_TREEBOOK
     m_type = Type_Treebook;
+#elif wxUSE_TOOLBOOK
+    m_type = Type_Toolbook;
+#elif wxUSE_AUI
+    m_type = Type_Aui;
 #else
-    #error "Don't use Notebook sample without any book enabled in wxWidgets build!"
+    m_type = Type_Simplebook;
 #endif
 
     m_orient = ID_ORIENT_DEFAULT;
     m_chkShowImages = true;
+    m_fixedWidth = false;
     m_multi = false;
+    m_noPageTheme = false;
+    m_buttonBar = false;
+    m_horzLayout = false;
 
     SetIcon(wxICON(sample));
 
@@ -244,15 +331,30 @@ MyFrame::MyFrame()
 #if wxUSE_TOOLBOOK
     menuType->AppendRadioItem(ID_BOOK_TOOLBOOK,   wxT("T&oolbook\tCtrl-5"));
 #endif
+#if wxUSE_AUI
+    menuType->AppendRadioItem(ID_BOOK_AUINOTEBOOK,   wxT("&AuiNotebook\tCtrl-6"));
+#endif
+    menuType->AppendRadioItem(ID_BOOK_SIMPLEBOOK, "&Simple book\tCtrl-7");
 
     menuType->Check(ID_BOOK_NOTEBOOK + m_type, true);
 
     wxMenu *menuOrient = new wxMenu;
-    menuOrient->AppendRadioItem(ID_ORIENT_DEFAULT, wxT("&Default\tCtrl-5"));
-    menuOrient->AppendRadioItem(ID_ORIENT_TOP,     wxT("&Top\tCtrl-6"));
-    menuOrient->AppendRadioItem(ID_ORIENT_BOTTOM,  wxT("&Bottom\tCtrl-7"));
-    menuOrient->AppendRadioItem(ID_ORIENT_LEFT,    wxT("&Left\tCtrl-8"));
-    menuOrient->AppendRadioItem(ID_ORIENT_RIGHT,   wxT("&Right\tCtrl-9"));
+    menuOrient->AppendRadioItem(ID_ORIENT_DEFAULT, wxT("&Default\tAlt-0"));
+    menuOrient->AppendRadioItem(ID_ORIENT_TOP,     wxT("&Top\tAlt-1"));
+    menuOrient->AppendRadioItem(ID_ORIENT_BOTTOM,  wxT("&Bottom\tAlt-2"));
+    menuOrient->AppendRadioItem(ID_ORIENT_LEFT,    wxT("&Left\tAlt-3"));
+    menuOrient->AppendRadioItem(ID_ORIENT_RIGHT,   wxT("&Right\tAlt-4"));
+
+    wxMenu *menuStyle = new wxMenu;
+#if wxUSE_NOTEBOOK
+    menuStyle->AppendCheckItem(ID_FIXEDWIDTH, wxT("&Fixed Width (wxNotebook)"));
+    menuStyle->AppendCheckItem(ID_MULTI, wxT("&Multiple lines (wxNotebook)"));
+    menuStyle->AppendCheckItem(ID_NOPAGETHEME, wxT("&No Page Theme (wxNotebook)"));
+#endif
+#if wxUSE_TOOLBOOK
+    menuStyle->AppendCheckItem(ID_BUTTONBAR, wxT("&Button Bar (wxToolbook)"));
+    menuStyle->AppendCheckItem(ID_HORZ_LAYOUT, wxT("&Horizontal layout (wxToolbook)"));
+#endif
 
     wxMenu *menuPageOperations = new wxMenu;
     menuPageOperations->Append(ID_ADD_PAGE, wxT("&Add page\tAlt-A"));
@@ -267,7 +369,11 @@ MyFrame::MyFrame()
     menuPageOperations->Append(ID_ADD_SUB_PAGE, wxT("Add s&ub page\tAlt-U"));
 #endif
     menuPageOperations->AppendSeparator();
-    menuPageOperations->Append(ID_GO_HOME, wxT("Go to the first page\tCtrl-F"));
+    menuPageOperations->Append(ID_CHANGE_SELECTION, wxT("&Change selection to 0\tCtrl-0"));
+    menuPageOperations->Append(ID_SET_SELECTION, wxT("&Set selection to 0\tShift-Ctrl-0"));
+    menuPageOperations->AppendSeparator();
+    menuPageOperations->Append(ID_GET_PAGE_SIZE, "Sho&w page size");
+    menuPageOperations->Append(ID_SET_PAGE_SIZE, "Set &page size");
 
     wxMenu *menuOperations = new wxMenu;
 #if wxUSE_HELP
@@ -279,11 +385,10 @@ MyFrame::MyFrame()
     menuFile->Append(wxID_ANY, wxT("&Type"), menuType, wxT("Type of control"));
     menuFile->Append(wxID_ANY, wxT("&Orientation"), menuOrient, wxT("Orientation of control"));
     menuFile->AppendCheckItem(ID_SHOW_IMAGES, wxT("&Show images\tAlt-S"));
-    menuFile->AppendCheckItem(ID_MULTI, wxT("&Multiple lines\tAlt-M"));
+    menuFile->Append(wxID_ANY, wxT("&Style"), menuStyle, wxT("Style of control"));
     menuFile->AppendSeparator();
     menuFile->Append(wxID_EXIT, wxT("E&xit"), wxT("Quits the application"));
     menuFile->Check(ID_SHOW_IMAGES, m_chkShowImages);
-    menuFile->Check(ID_MULTI, m_multi);
 
     wxMenuBar *menuBar = new wxMenuBar;
     menuBar->Append(menuFile, wxT("&File"));
@@ -328,11 +433,11 @@ MyFrame::MyFrame()
     RecreateBook();
 
     m_panel->SetSizer(m_sizerFrame);
+    m_panel->Layout();
 
-    m_sizerFrame->Fit(this);
-    m_sizerFrame->SetSizeHints(this);
-
-    Centre(wxBOTH);
+    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
+    sizer->Add(m_panel, wxSizerFlags(1).Expand());
+    SetSizerAndFit(sizer);
 }
 
 MyFrame::~MyFrame()
@@ -378,28 +483,29 @@ MyFrame::~MyFrame()
     #define CASE_TOOLBOOK(x)
 #endif
 
-#define DISPATCH_ON_TYPE(before, nb, lb, cb, tb, toolb, after)                       \
+#if wxUSE_AUI
+    #define CASE_AUINOTEBOOK(x) case Type_AuiNotebook: x; break;
+#else
+    #define CASE_AUINOTEBOOK(x)
+#endif
+
+#define CASE_SIMPLEBOOK(x) case Type_Simplebook: x; break;
+
+#define DISPATCH_ON_TYPE(before, nb, lb, cb, tb, toolb, aui, sb, after)       \
     switch ( m_type )                                                         \
     {                                                                         \
         CASE_NOTEBOOK(before nb after)                                        \
         CASE_LISTBOOK(before lb after)                                        \
         CASE_CHOICEBOOK(before cb after)                                      \
         CASE_TREEBOOK(before tb after)                                        \
-        CASE_TOOLBOOK(before toolb after)                                        \
+        CASE_TOOLBOOK(before toolb after)                                     \
+        CASE_AUINOTEBOOK(before aui after)                                    \
+        CASE_SIMPLEBOOK(before sb after)                                      \
                                                                               \
         default:                                                              \
-            wxFAIL_MSG( _T("unknown book control type") );                    \
+            wxFAIL_MSG( wxT("unknown book control type") );                   \
     }
 
-int MyFrame::TranslateBookFlag(int nb, int lb, int chb, int tbk, int toolbk) const
-{
-    int flag = 0;
-
-    DISPATCH_ON_TYPE(flag =, nb,  lb,  chb,  tbk, toolbk, + 0);
-
-    return flag;
-}
-
 void MyFrame::RecreateBook()
 {
     int flags;
@@ -425,8 +531,20 @@ void MyFrame::RecreateBook()
             flags = wxBK_DEFAULT;
     }
 
+#if wxUSE_NOTEBOOK
+    if ( m_fixedWidth && m_type == Type_Notebook )
+        flags |= wxNB_FIXEDWIDTH;
     if ( m_multi && m_type == Type_Notebook )
         flags |= wxNB_MULTILINE;
+    if ( m_noPageTheme && m_type == Type_Notebook )
+        flags |= wxNB_NOPAGETHEME;
+#endif
+#if wxUSE_TOOLBOOK
+    if ( m_buttonBar && m_type == Type_Toolbook )
+        flags |= wxTBK_BUTTONBAR;
+    if ( m_horzLayout && m_type == Type_Toolbook )
+        flags |= wxTBK_HORZ_LAYOUT;
+#endif
 
     wxBookCtrlBase *oldBook = m_bookCtrl;
 
@@ -438,6 +556,8 @@ void MyFrame::RecreateBook()
                          wxChoicebook,
                          wxTreebook,
                          wxToolbook,
+                         wxAuiNotebook,
+                         wxSimplebook,
                      (m_panel, wxID_ANY, wxDefaultPosition, wxDefaultSize, flags));
 
     if ( !m_bookCtrl )
@@ -445,7 +565,8 @@ void MyFrame::RecreateBook()
 
     m_bookCtrl->Hide();
 
-    if ( m_chkShowImages )
+    // wxToolbook doesn't work without icons so always use them for it.
+    if ( m_chkShowImages || m_type == Type_Toolbook )
     {
         m_bookCtrl->SetImageList(m_imageList);
     }
@@ -505,58 +626,35 @@ void MyFrame::RecreateBook()
     m_sizerFrame->Layout();
 }
 
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
-    // File menu
-    EVT_MENU_RANGE(ID_BOOK_NOTEBOOK, ID_BOOK_MAX, MyFrame::OnType)
-    EVT_MENU_RANGE(ID_ORIENT_DEFAULT, ID_ORIENT_MAX, MyFrame::OnOrient)
-    EVT_MENU(ID_SHOW_IMAGES, MyFrame::OnShowImages)
-    EVT_MENU(ID_MULTI, MyFrame::OnMulti)
-    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
+void MyFrame::AddFlagStrIfFlagPresent(wxString & flagStr, long flags, long flag,
+                                      const wxChar * flagName) const
+{
+    if( (flags & flag) == flag )
+    {
+        if( !flagStr.empty() )
+            flagStr += wxT(" | ");
+        flagStr += flagName;
+    }
+}
 
-    // Operations menu
-    EVT_MENU(ID_ADD_PAGE, MyFrame::OnAddPage)
-    EVT_MENU(ID_ADD_PAGE_NO_SELECT, MyFrame::OnAddPageNoSelect)
-    EVT_MENU(ID_INSERT_PAGE, MyFrame::OnInsertPage)
-    EVT_MENU(ID_DELETE_CUR_PAGE, MyFrame::OnDeleteCurPage)
-    EVT_MENU(ID_DELETE_LAST_PAGE, MyFrame::OnDeleteLastPage)
-    EVT_MENU(ID_NEXT_PAGE, MyFrame::OnNextPage)
-    EVT_MENU(ID_GO_HOME, MyFrame::OnGoHome)
+wxPanel *MyFrame::CreateNewPage() const
+{
+    wxPanel *panel = new wxPanel(m_bookCtrl, wxID_ANY );
 
 #if wxUSE_HELP
-    EVT_MENU(ID_CONTEXT_HELP, MyFrame::OnContextHelp)
-#endif // wxUSE_HELP
-    EVT_MENU(ID_HITTEST, MyFrame::OnHitTest)
-
-    // Book controls
-#if wxUSE_NOTEBOOK
-    EVT_NOTEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnNotebook)
-    EVT_NOTEBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnNotebook)
-#endif
-#if wxUSE_LISTBOOK
-    EVT_LISTBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnListbook)
-    EVT_LISTBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnListbook)
-#endif
-#if wxUSE_CHOICEBOOK
-    EVT_CHOICEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnChoicebook)
-    EVT_CHOICEBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnChoicebook)
-#endif
-#if wxUSE_TREEBOOK
-    EVT_TREEBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnTreebook)
-    EVT_TREEBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnTreebook)
-
-    EVT_MENU(ID_ADD_SUB_PAGE, MyFrame::OnAddSubPage)
-    EVT_MENU(ID_ADD_PAGE_BEFORE, MyFrame::OnAddPageBefore)
-    EVT_UPDATE_UI_RANGE(ID_ADD_PAGE_BEFORE, ID_ADD_SUB_PAGE,
-                            MyFrame::OnUpdateTreeMenu)
-#endif
-#if wxUSE_TOOLBOOK
-    EVT_TOOLBOOK_PAGE_CHANGED(wxID_ANY, MyFrame::OnToolbook)
-    EVT_TOOLBOOK_PAGE_CHANGING(wxID_ANY, MyFrame::OnToolbook)
+    panel->SetHelpText( wxT( "Panel with \"First\" and \"Second\" buttons" ) );
 #endif
 
-    // Update title in idle time
-    EVT_IDLE(MyFrame::OnIdle)
-END_EVENT_TABLE()
+    (void) new wxButton(panel, wxID_ANY, wxT("First button"), wxPoint(10, 30));
+    (void) new wxButton(panel, wxID_ANY, wxT("Second button"), wxPoint(150, 30));
+
+    return panel;
+}
+
+
+//-----------------------------------------------------------------------------
+// MyFrame - event handlers
+//-----------------------------------------------------------------------------
 
 #if wxUSE_HELP
 
@@ -568,16 +666,6 @@ void MyFrame::OnContextHelp(wxCommandEvent& WXUNUSED(event))
 
 #endif // wxUSE_HELP
 
-void MyFrame::AddFlagStrIfFlagPresent(wxString & flagStr, long flags, long flag, const wxChar * flagName) const
-{
-    if( (flags & flag) == flag )
-    {
-        if( !flagStr.empty() )
-            flagStr += _T(" | ");
-        flagStr += flagName;
-    }
-}
-
 void MyFrame::OnHitTest(wxCommandEvent& WXUNUSED(event))
 {
     wxBookCtrlBase * book = GetCurrentBook();
@@ -588,10 +676,10 @@ void MyFrame::OnHitTest(wxCommandEvent& WXUNUSED(event))
 
     wxString flagsStr;
 
-    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_NOWHERE, _T("wxBK_HITTEST_NOWHERE") );
-    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_ONICON,  _T("wxBK_HITTEST_ONICON") );
-    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_ONLABEL, _T("wxBK_HITTEST_ONLABEL") );
-    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_ONPAGE,  _T("wxBK_HITTEST_ONPAGE") );
+    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_NOWHERE, wxT("wxBK_HITTEST_NOWHERE") );
+    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_ONICON,  wxT("wxBK_HITTEST_ONICON") );
+    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_ONLABEL, wxT("wxBK_HITTEST_ONLABEL") );
+    AddFlagStrIfFlagPresent( flagsStr, flags, wxBK_HITTEST_ONPAGE,  wxT("wxBK_HITTEST_ONPAGE") );
 
     wxLogMessage(wxT("HitTest at (%d,%d): %d: %s"),
                  pt.x,
@@ -602,7 +690,7 @@ void MyFrame::OnHitTest(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnType(wxCommandEvent& event)
 {
-    m_type = wx_static_cast(BookType, event.GetId() - ID_BOOK_NOTEBOOK);
+    m_type = static_cast<BookType>(event.GetId() - ID_BOOK_NOTEBOOK);
 
     if ( m_bookCtrl )
         m_sizerFrame->Hide(m_bookCtrl);
@@ -619,7 +707,6 @@ void MyFrame::OnUpdateTreeMenu(wxUpdateUIEvent& event)
 
 #endif // wxUSE_TREEBOOK
 
-
 void MyFrame::OnOrient(wxCommandEvent& event)
 {
     m_orient = event.GetId();
@@ -634,12 +721,21 @@ void MyFrame::OnShowImages(wxCommandEvent& event)
     m_sizerFrame->Layout();
 }
 
-void MyFrame::OnMulti(wxCommandEvent& event)
+void MyFrame::OnStyle(wxCommandEvent& event)
 {
-    m_multi = event.IsChecked();
+    bool checked = event.IsChecked();
+    switch (event.GetId())
+    {
+        case ID_FIXEDWIDTH:  m_fixedWidth = checked;  break;
+        case ID_MULTI:       m_multi = checked;       break;
+        case ID_NOPAGETHEME: m_noPageTheme = checked; break;
+        case ID_BUTTONBAR:   m_buttonBar = checked;   break;
+        case ID_HORZ_LAYOUT: m_horzLayout = checked;  break;
+        default: break; // avoid compiler warning
+    }
+
     RecreateBook();
     m_sizerFrame->Layout();
-    wxLogMessage(_T("Multiline setting works only in wxNotebook."));
 }
 
 void MyFrame::OnExit(wxCommandEvent& WXUNUSED(event))
@@ -647,20 +743,6 @@ void MyFrame::OnExit(wxCommandEvent& WXUNUSED(event))
     Close();
 }
 
-wxPanel *MyFrame::CreateNewPage() const
-{
-    wxPanel *panel = new wxPanel(m_bookCtrl, wxID_ANY );
-
-#if wxUSE_HELP
-    panel->SetHelpText( wxT( "Panel with \"First\" and \"Second\" buttons" ) );
-#endif
-
-    (void) new wxButton(panel, wxID_ANY, wxT("First button"), wxPoint(10, 10));
-    (void) new wxButton(panel, wxID_ANY, wxT("Second button"), wxPoint(50, 100));
-
-    return panel;
-}
-
 void MyFrame::OnAddPage(wxCommandEvent& WXUNUSED(event))
 {
     wxBookCtrlBase *currBook = GetCurrentBook();
@@ -706,7 +788,7 @@ void MyFrame::OnAddSubPage(wxCommandEvent& WXUNUSED(event))
         const int selPos = currBook->GetSelection();
         if ( selPos == wxNOT_FOUND )
         {
-            wxLogError(_T("Select the parent page first!"));
+            wxLogError(wxT("Select the parent page first!"));
             return;
         }
 
@@ -734,7 +816,7 @@ void MyFrame::OnAddPageBefore(wxCommandEvent& WXUNUSED(event))
         const int selPos = currBook->GetSelection();
         if ( selPos == wxNOT_FOUND )
         {
-            wxLogError(_T("Select the parent page first!"));
+            wxLogError(wxT("Select the parent page first!"));
             return;
         }
 
@@ -810,16 +892,47 @@ void MyFrame::OnNextPage(wxCommandEvent& WXUNUSED(event))
     }
 }
 
-void MyFrame::OnGoHome(wxCommandEvent& WXUNUSED(event))
+void MyFrame::OnChangeSelection(wxCommandEvent& WXUNUSED(event))
 {
     wxBookCtrlBase *currBook = GetCurrentBook();
 
     if ( currBook )
-    {
-        // ChangeSelection shouldn't send any events, SetSelection() should
         currBook->ChangeSelection(0);
-        //currBook->SetSelection(0);
-    }
+}
+
+void MyFrame::OnSetSelection(wxCommandEvent& WXUNUSED(event))
+{
+    wxBookCtrlBase *currBook = GetCurrentBook();
+
+    if ( currBook )
+        currBook->SetSelection(0);
+}
+
+void MyFrame::OnGetPageSize(wxCommandEvent& WXUNUSED(event))
+{
+    wxBookCtrlBase* const currBook = GetCurrentBook();
+    if ( !currBook )
+        return;
+
+    const wxSize sizePage = currBook->GetPage(0)->GetSize();
+    const wxSize sizeBook = currBook->GetSize();
+
+    wxLogMessage("Page size is (%d, %d), book size (%d, %d)",
+                 sizePage.x, sizePage.y,
+                 sizeBook.x, sizeBook.y);
+}
+
+void MyFrame::OnSetPageSize(wxCommandEvent& WXUNUSED(event))
+{
+    wxBookCtrlBase* const currBook = GetCurrentBook();
+    if ( !currBook )
+        return;
+
+    const wxSize sizePage(300, 300);
+    currBook->SetPageSize(sizePage);
+
+    wxLogMessage("Page size set to (%d, %d)",
+                 sizePage.x, sizePage.y);
 }
 
 void MyFrame::OnIdle( wxIdleEvent& WXUNUSED(event) )
@@ -863,39 +976,46 @@ void MyFrame::OnBookCtrl(wxBookCtrlBaseEvent& event)
     {
 #if wxUSE_NOTEBOOK
         {
-            wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED,
-            wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING,
-            _T("wxNotebook")
+            wxEVT_NOTEBOOK_PAGE_CHANGED,
+            wxEVT_NOTEBOOK_PAGE_CHANGING,
+            wxT("wxNotebook")
         },
 #endif // wxUSE_NOTEBOOK
 #if wxUSE_LISTBOOK
         {
-            wxEVT_COMMAND_LISTBOOK_PAGE_CHANGED,
-            wxEVT_COMMAND_LISTBOOK_PAGE_CHANGING,
-            _T("wxListbook")
+            wxEVT_LISTBOOK_PAGE_CHANGED,
+            wxEVT_LISTBOOK_PAGE_CHANGING,
+            wxT("wxListbook")
         },
 #endif // wxUSE_LISTBOOK
 #if wxUSE_CHOICEBOOK
         {
-            wxEVT_COMMAND_CHOICEBOOK_PAGE_CHANGED,
-            wxEVT_COMMAND_CHOICEBOOK_PAGE_CHANGING,
-            _T("wxChoicebook")
+            wxEVT_CHOICEBOOK_PAGE_CHANGED,
+            wxEVT_CHOICEBOOK_PAGE_CHANGING,
+            wxT("wxChoicebook")
         },
 #endif // wxUSE_CHOICEBOOK
 #if wxUSE_TREEBOOK
         {
-            wxEVT_COMMAND_TREEBOOK_PAGE_CHANGED,
-            wxEVT_COMMAND_TREEBOOK_PAGE_CHANGING,
-            _T("wxTreebook")
+            wxEVT_TREEBOOK_PAGE_CHANGED,
+            wxEVT_TREEBOOK_PAGE_CHANGING,
+            wxT("wxTreebook")
         },
 #endif // wxUSE_TREEBOOK
 #if wxUSE_TOOLBOOK
         {
-            wxEVT_COMMAND_TOOLBOOK_PAGE_CHANGED,
-            wxEVT_COMMAND_TOOLBOOK_PAGE_CHANGING,
-            _T("wxToolbook")
+            wxEVT_TOOLBOOK_PAGE_CHANGED,
+            wxEVT_TOOLBOOK_PAGE_CHANGING,
+            wxT("wxToolbook")
         },
 #endif // wxUSE_TOOLBOOK
+#if wxUSE_AUI
+        {
+            wxEVT_AUINOTEBOOK_PAGE_CHANGED,
+            wxEVT_AUINOTEBOOK_PAGE_CHANGING,
+            wxT("wxAuiNotebook")
+        },
+#endif // wxUSE_AUI
     };
 
 
@@ -903,6 +1023,12 @@ void MyFrame::OnBookCtrl(wxBookCtrlBaseEvent& event)
              nameControl,
              veto;
     const wxEventType eventType = event.GetEventType();
+
+    // NB: can't use wxStaticCast here as wxBookCtrlBase is not in
+    //     wxRTTI
+    const wxBookCtrlBase * const
+        book = static_cast<wxBookCtrlBase *>(event.GetEventObject());
+
     for ( size_t n = 0; n < WXSIZEOF(events); n++ )
     {
         const EventInfo& ei = events[n];
@@ -914,10 +1040,6 @@ void MyFrame::OnBookCtrl(wxBookCtrlBaseEvent& event)
         {
             const int idx = event.GetOldSelection();
 
-            // NB: can't use wxStaticCast here as wxBookCtrlBase is not in
-            //     wxRTTI
-            const wxBookCtrlBase * const
-                book = wx_static_cast(wxBookCtrlBase *, event.GetEventObject());
             if ( idx != wxNOT_FOUND &&
                     book && book->GetPageText(idx) == VETO_PAGE_NAME )
             {
@@ -931,7 +1053,7 @@ void MyFrame::OnBookCtrl(wxBookCtrlBaseEvent& event)
                      ) != wxYES )
                 {
                     event.Veto();
-                    veto = _T(" (vetoed)");
+                    veto = wxT(" (vetoed)");
                 }
             }
 
@@ -948,13 +1070,14 @@ void MyFrame::OnBookCtrl(wxBookCtrlBaseEvent& event)
 
     static int s_num = 0;
 
-    wxLogMessage(wxT("Event #%d: %s: %s (%d) new sel %d, old %d%s"),
+    wxLogMessage(wxT("Event #%d: %s: %s (%d) new sel %d, old %d, current %d%s"),
                  ++s_num,
                  nameControl.c_str(),
                  nameEvent.c_str(),
                  eventType,
                  event.GetSelection(),
                  event.GetOldSelection(),
+                 book->GetSelection(),
                  veto.c_str());
 
 #if USE_LOG
