@@ -315,9 +315,9 @@ wxAppTraits *wxAppConsoleBase::GetTraitsIfExists()
 wxAppTraits& wxAppConsoleBase::GetValidTraits()
 {
     static wxConsoleAppTraits s_traitsConsole;
-    wxAppTraits* const traits = wxTheApp ? wxTheApp->GetTraits() : NULL;
+    wxAppTraits* const traits = (wxTheApp ? wxTheApp->GetTraits() : NULL);
 
-    return traits ? *traits : s_traitsConsole;
+    return *(traits ? traits : &s_traitsConsole);
 }
 
 // ----------------------------------------------------------------------------
@@ -638,8 +638,12 @@ void wxAppConsoleBase::OnUnhandledException()
 #if wxUSE_STL
     catch ( std::exception& e )
     {
+#ifdef wxNO_RTTI
+        what.Printf("std::exception, what() = \"%s\"", e.what());
+#else
         what.Printf("std::exception of type \"%s\", what() = \"%s\"",
                     typeid(e).name(), e.what());
+#endif
     }
 #endif // wxUSE_STL
     catch ( ... )
