@@ -201,12 +201,15 @@ void wxScrollBar::SetThumbPosition( int viewStart )
 
 void wxScrollBar::SetScrollbar(int position, int thumbSize, int range, int pageSize, bool)
 {
-    if (range == 0)
+    if (range <= 0)
     {
         // GtkRange requires upper > lower
         range =
+        pageSize =
         thumbSize = 1;
     }
+    else if (pageSize <= 0)
+        pageSize = 1;
     g_signal_handlers_block_by_func(m_widget, (void*)gtk_value_changed, this);
     GtkRange* widget = GTK_RANGE(m_widget);
     GtkAdjustment* adj = gtk_range_get_adjustment(widget);
