@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: statbox.cpp 52851 2008-03-27 13:03:02Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -33,6 +32,7 @@
     #include "wx/dcclient.h"
     #include "wx/dcmemory.h"
     #include "wx/image.h"
+    #include "wx/sizer.h"
 #endif
 
 #include "wx/notebook.h"
@@ -41,6 +41,7 @@
 #include "wx/msw/uxtheme.h"
 #include "wx/msw/private.h"
 #include "wx/msw/missing.h"
+#include "wx/msw/dc.h"
 
 // the values coincide with those in tmschema.h
 #define BP_GROUPBOX 4
@@ -53,58 +54,6 @@
 // wxWin macros
 // ----------------------------------------------------------------------------
 
-#if wxUSE_EXTENDED_RTTI
-WX_DEFINE_FLAGS( wxStaticBoxStyle )
-
-wxBEGIN_FLAGS( wxStaticBoxStyle )
-    // new style border flags, we put them first to
-    // use them for streaming out
-    wxFLAGS_MEMBER(wxBORDER_SIMPLE)
-    wxFLAGS_MEMBER(wxBORDER_SUNKEN)
-    wxFLAGS_MEMBER(wxBORDER_DOUBLE)
-    wxFLAGS_MEMBER(wxBORDER_RAISED)
-    wxFLAGS_MEMBER(wxBORDER_STATIC)
-    wxFLAGS_MEMBER(wxBORDER_NONE)
-
-    // old style border flags
-    wxFLAGS_MEMBER(wxSIMPLE_BORDER)
-    wxFLAGS_MEMBER(wxSUNKEN_BORDER)
-    wxFLAGS_MEMBER(wxDOUBLE_BORDER)
-    wxFLAGS_MEMBER(wxRAISED_BORDER)
-    wxFLAGS_MEMBER(wxSTATIC_BORDER)
-    wxFLAGS_MEMBER(wxBORDER)
-
-    // standard window styles
-    wxFLAGS_MEMBER(wxTAB_TRAVERSAL)
-    wxFLAGS_MEMBER(wxCLIP_CHILDREN)
-    wxFLAGS_MEMBER(wxTRANSPARENT_WINDOW)
-    wxFLAGS_MEMBER(wxWANTS_CHARS)
-    wxFLAGS_MEMBER(wxFULL_REPAINT_ON_RESIZE)
-    wxFLAGS_MEMBER(wxALWAYS_SHOW_SB )
-    wxFLAGS_MEMBER(wxVSCROLL)
-    wxFLAGS_MEMBER(wxHSCROLL)
-
-wxEND_FLAGS( wxStaticBoxStyle )
-
-IMPLEMENT_DYNAMIC_CLASS_XTI(wxStaticBox, wxControl,"wx/statbox.h")
-
-wxBEGIN_PROPERTIES_TABLE(wxStaticBox)
-    wxPROPERTY( Label,wxString, SetLabel, GetLabel, wxString() , 0 /*flags*/ , wxT("Helpstring") , wxT("group"))
-    wxPROPERTY_FLAGS( WindowStyle , wxStaticBoxStyle , long , SetWindowStyleFlag , GetWindowStyleFlag , EMPTY_MACROVALUE, 0 /*flags*/ , wxT("Helpstring") , wxT("group")) // style
-/*
-    TODO PROPERTIES :
-        label
-*/
-wxEND_PROPERTIES_TABLE()
-
-wxBEGIN_HANDLERS_TABLE(wxStaticBox)
-wxEND_HANDLERS_TABLE()
-
-wxCONSTRUCTOR_6( wxStaticBox , wxWindow* , Parent , wxWindowID , Id , wxString , Label , wxPoint , Position , wxSize , Size , long , WindowStyle )
-#else
-IMPLEMENT_DYNAMIC_CLASS(wxStaticBox, wxControl)
-#endif
-
 // ============================================================================
 // implementation
 // ============================================================================
@@ -127,22 +76,20 @@ bool wxStaticBox::Create(wxWindow *parent,
     if ( !MSWCreateControl(wxT("BUTTON"), label, pos, size) )
         return false;
 
-    // Always use LTR layout. Otherwise, the label would be mirrored.
-    SetLayoutDirection(wxLayout_LeftToRight);
-
 #ifndef __WXWINCE__
     if (!wxSystemOptions::IsFalse(wxT("msw.staticbox.optimized-paint")))
+    {
         Connect(wxEVT_PAINT, wxPaintEventHandler(wxStaticBox::OnPaint));
+
+        // Our OnPaint() completely erases our background, so don't do it in
+        // WM_ERASEBKGND too to avoid flicker.
+        SetBackgroundStyle(wxBG_STYLE_PAINT);
+    }
 #endif // !__WXWINCE__
 
     return true;
 }
 
-wxBorder wxStaticBox::GetDefaultBorder() const
-{
-    return wxBORDER_NONE;
-}
-
 WXDWORD wxStaticBox::MSWGetStyle(long style, WXDWORD *exstyle) const
 {
     long styleWin = wxStaticBoxBase::MSWGetStyle(style, exstyle);
@@ -154,26 +101,27 @@ WXDWORD wxStaticBox::MSWGetStyle(long style, WXDWORD *exstyle) const
     if ( exstyle )
     {
 #ifndef __WXWINCE__
+        // We may have children inside this static box, so use this style for
+        // TAB navigation to work if we ever use IsDialogMessage() to implement
+        // it (currently we don't because it's too buggy and implement TAB
+        // navigation ourselves, but this could change in the future).
+        *exstyle |= WS_EX_CONTROLPARENT;
+
         if (wxSystemOptions::IsFalse(wxT("msw.staticbox.optimized-paint")))
-            *exstyle = WS_EX_TRANSPARENT;
-        else
+            *exstyle |= WS_EX_TRANSPARENT;
 #endif
-            *exstyle = 0;
     }
 
     styleWin |= BS_GROUPBOX;
 
-    if ( wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft )
-    {
-        // Make sure label is on the right
-        styleWin |= BS_RIGHT;
-    }
-
     return styleWin;
 }
 
 wxSize wxStaticBox::DoGetBestSize() const
 {
+    wxSize best;
+
+    // Calculate the size needed by the label
     int cx, cy;
     wxGetCharSize(GetHWND(), &cx, &cy, GetFont());
 
@@ -183,8 +131,19 @@ wxSize wxStaticBox::DoGetBestSize() const
     wBox += 3*cx;
     int hBox = EDIT_HEIGHT_FROM_CHAR_HEIGHT(cy);
 
-    wxSize best(wBox, hBox);
-    CacheBestSize(best);
+    // If there is a sizer then the base best size is the sizer's minimum
+    if (GetSizer() != NULL)
+    {
+        wxSize cm(GetSizer()->CalcMin());
+        best = ClientToWindowSize(cm);
+        // adjust for a long label if needed
+        best.x = wxMax(best.x, wBox);
+    }
+    // otherwise the best size falls back to the label size
+    else
+    {
+        best = wxSize(wBox, hBox);
+    }
     return best;
 }
 
@@ -241,6 +200,24 @@ WXLRESULT wxStaticBox::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lPar
         return 0;
     }
 
+    if ( nMsg == WM_UPDATEUISTATE )
+    {
+        // DefWindowProc() redraws just the static box text when it gets this
+        // message and it does it using the standard (blue in standard theme)
+        // colour and not our own label colour that we use in PaintForeground()
+        // resulting in the label mysteriously changing the colour when e.g.
+        // "Alt" is pressed anywhere in the window, see #12497.
+        //
+        // To avoid this we simply refresh the window forcing our own code
+        // redrawing the label in the correct colour to be called. This is
+        // inefficient but there doesn't seem to be anything else we can do.
+        //
+        // Notice that the problem is XP-specific and doesn't arise under later
+        // systems.
+        if ( m_hasFgCol && wxGetWinVersion() == wxWinVersion_XP )
+            Refresh();
+    }
+
     return wxControl::MSWWindowProc(nMsg, wParam, lParam);
 }
 
@@ -263,7 +240,7 @@ SubtractRectFromRgn(HRGN hrgn, int left, int top, int right, int bottom)
     AutoHRGN hrgnRect(::CreateRectRgn(left, top, right, bottom));
     if ( !hrgnRect )
     {
-        wxLogLastError(_T("CreateRectRgn()"));
+        wxLogLastError(wxT("CreateRectRgn()"));
         return;
     }
 
@@ -291,15 +268,36 @@ void wxStaticBox::MSWGetRegionWithoutSelf(WXHRGN hRgn, int w, int h)
     SubtractRectFromRgn(hrgn, w - border, 0, w, h);
 }
 
+namespace {
+RECT AdjustRectForRtl(wxLayoutDirection dir, RECT const& childRect, RECT const& boxRect) {
+    RECT ret = childRect;
+    if( dir == wxLayout_RightToLeft ) {
+        // The clipping region too is mirrored in RTL layout.
+        // We need to mirror screen coordinates relative to static box window priot to
+        // intersecting with region.
+        ret.right = boxRect.right - childRect.left - boxRect.left;
+        ret.left = boxRect.right - childRect.right - boxRect.left;
+    }
+
+    return ret;
+}
+}
+
 WXHRGN wxStaticBox::MSWGetRegionWithoutChildren()
 {
-    RECT rc;
-    ::GetWindowRect(GetHwnd(), &rc);
-    HRGN hrgn = ::CreateRectRgn(rc.left, rc.top, rc.right + 1, rc.bottom + 1);
+    RECT boxRc;
+    ::GetWindowRect(GetHwnd(), &boxRc);
+    HRGN hrgn = ::CreateRectRgn(boxRc.left, boxRc.top, boxRc.right + 1, boxRc.bottom + 1);
     bool foundThis = false;
 
-    // iterate over all child windows (not just wxWindows but all windows)
-    for ( HWND child = ::GetWindow(GetHwndOf(GetParent()), GW_CHILD);
+    // Iterate over all sibling windows as in the old wxWidgets API the
+    // controls appearing inside the static box were created as its siblings
+    // and not children. This is now deprecated but should still work.
+    //
+    // Also notice that we must iterate over all windows, not just all
+    // wxWindows, as there may be composite windows etc.
+    HWND child;
+    for ( child = ::GetWindow(GetHwndOf(GetParent()), GW_CHILD);
           child;
           child = ::GetWindow(child, GW_HWNDNEXT) )
     {
@@ -328,7 +326,9 @@ WXHRGN wxStaticBox::MSWGetRegionWithoutChildren()
                 continue;
         }
 
+        RECT rc;
         ::GetWindowRect(child, &rc);
+        rc = AdjustRectForRtl(GetLayoutDirection(), rc, boxRc );
         if ( ::RectInRegion(hrgn, &rc) )
         {
             // need to remove WS_CLIPSIBLINGS from all sibling windows
@@ -351,6 +351,25 @@ WXHRGN wxStaticBox::MSWGetRegionWithoutChildren()
         }
     }
 
+    // Also iterate over all children of the static box, we need to clip them
+    // out as well.
+    for ( child = ::GetWindow(GetHwnd(), GW_CHILD);
+          child;
+          child = ::GetWindow(child, GW_HWNDNEXT) )
+    {
+        if ( !::IsWindowVisible(child) )
+        {
+            // if the window isn't visible then it doesn't need clipped
+            continue;
+        }
+
+        RECT rc;
+        ::GetWindowRect(child, &rc);
+        rc = AdjustRectForRtl(GetLayoutDirection(), rc, boxRc );
+        AutoHRGN hrgnChild(::CreateRectRgnIndirect(&rc));
+        ::CombineRgn(hrgn, hrgn, hrgnChild, RGN_DIFF);
+    }
+
     return (WXHRGN)hrgn;
 }
 
@@ -367,25 +386,27 @@ void wxStaticBox::PaintBackground(wxDC& dc, const RECT& rc)
     //     we did it
     //  3. this is backwards compatible behaviour and some people rely on it,
     //     see http://groups.google.com/groups?selm=4252E932.3080801%40able.es
-    wxWindow *parent = GetParent();
-    HBRUSH hbr = (HBRUSH)parent->MSWGetBgBrush(dc.GetHDC(), GetHWND());
+    wxMSWDCImpl *impl = (wxMSWDCImpl*) dc.GetImpl();
+    HBRUSH hbr = MSWGetBgBrush(impl->GetHDC());
 
     // if there is no special brush for painting this control, just use the
     // solid background colour
     wxBrush brush;
     if ( !hbr )
     {
-        brush = wxBrush(parent->GetBackgroundColour());
+        brush = wxBrush(GetParent()->GetBackgroundColour());
         hbr = GetHbrushOf(brush);
     }
 
-    ::FillRect(GetHdcOf(dc), &rc, hbr);
+    ::FillRect(GetHdcOf(*impl), &rc, hbr);
 }
 
-void wxStaticBox::PaintForeground(wxDC& dc, const RECT& rc)
+void wxStaticBox::PaintForeground(wxDC& dc, const RECT&)
 {
-    MSWDefWindowProc(WM_PAINT, (WPARAM)GetHdcOf(dc), 0);
+    wxMSWDCImpl *impl = (wxMSWDCImpl*) dc.GetImpl();
+    MSWDefWindowProc(WM_PAINT, (WPARAM)GetHdcOf(*impl), 0);
 
+#if wxUSE_UXTHEME
     // when using XP themes, neither setting the text colour nor transparent
     // background mode doesn't change anything: the static box def window proc
     // still draws the label in its own colours, so we need to redraw the text
@@ -393,13 +414,9 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT& rc)
     if ( m_hasFgCol && wxUxThemeEngine::GetIfActive() )
     {
         // draw over the text in default colour in our colour
-        HDC hdc = GetHdcOf(dc);
+        HDC hdc = GetHdcOf(*impl);
         ::SetTextColor(hdc, GetForegroundColour().GetPixel());
 
-        const bool rtl = wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft;
-        if ( rtl )
-            ::SetTextAlign(hdc, TA_RTLREADING | TA_RIGHT);
-
         // Get dimensions of the label
         const wxString label = GetLabel();
 
@@ -415,11 +432,7 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT& rc)
             wxUxThemeHandle hTheme(this, L"BUTTON");
             if ( hTheme )
             {
-                // GetThemeFont() expects its parameter to be LOGFONTW and not
-                // LOGFONTA even in ANSI programs and will happily corrupt
-                // memory after the struct end if we pass a LOGFONTA (which is
-                // smaller) to it!
-                LOGFONTW lfw;
+                wxUxThemeFont themeFont;
                 if ( wxUxThemeEngine::Get()->GetThemeFont
                                              (
                                                 hTheme,
@@ -427,24 +440,10 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT& rc)
                                                 BP_GROUPBOX,
                                                 GBS_NORMAL,
                                                 TMT_FONT,
-                                                (LOGFONT *)&lfw
+                                                themeFont.GetPtr()
                                              ) == S_OK )
                 {
-#if wxUSE_UNICODE
-                    // ok, no conversion necessary
-                    const LOGFONT& lf = lfw;
-#else // !wxUSE_UNICODE
-                    // most of the fields are the same in LOGFONTA and LOGFONTW
-                    LOGFONT lf;
-                    memcpy(&lf, &lfw, sizeof(lf));
-
-                    // but the face name must be converted
-                    WideCharToMultiByte(CP_ACP, 0, lfw.lfFaceName, -1,
-                                        lf.lfFaceName, sizeof(lf.lfFaceName),
-                                        NULL, NULL);
-#endif // wxUSE_UNICODE/!wxUSE_UNICODE
-
-                    font.Init(lf);
+                    font.Init(themeFont.GetLOGFONT());
                     if ( font )
                         selFont.Init(hdc, font);
                 }
@@ -465,18 +464,9 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT& rc)
         // FIXME: value of x is hardcoded as this is what it is on my system,
         //        no idea if it's true everywhere
         RECT dimensions = {0, 0, 0, y};
-        if ( !rtl )
-        {
-            x = 9;
-            dimensions.left = x;
-            dimensions.right = x + width;
-        }
-        else
-        {
-            x = rc.right - 7;
-            dimensions.left = x - width;
-            dimensions.right = x;
-        }
+        x = 9;
+        dimensions.left = x;
+        dimensions.right = x + width;
 
         // need to adjust the rectangle to cover all the label background
         dimensions.left -= 2;
@@ -489,36 +479,43 @@ void wxStaticBox::PaintForeground(wxDC& dc, const RECT& rc)
             // the label: this is consistent with the behaviour under pre-XP
             // systems (i.e. without visual themes) and generally makes sense
             wxBrush brush = wxBrush(GetBackgroundColour());
-            ::FillRect(GetHdcOf(dc), &dimensions, GetHbrushOf(brush));
+            ::FillRect(hdc, &dimensions, GetHbrushOf(brush));
         }
         else // paint parent background
         {
             PaintBackground(dc, dimensions);
         }
 
+        UINT drawTextFlags = DT_SINGLELINE | DT_VCENTER;
+
+        // determine the state of UI queues to draw the text correctly under XP
+        // and later systems
+        static const bool isXPorLater = wxGetWinVersion() >= wxWinVersion_XP;
+        if ( isXPorLater )
+        {
+            if ( ::SendMessage(GetHwnd(), WM_QUERYUISTATE, 0, 0) &
+                    UISF_HIDEACCEL )
+            {
+                drawTextFlags |= DT_HIDEPREFIX;
+            }
+        }
+
         // now draw the text
-        if ( !rtl )
-        {
-            RECT rc2 = { x, 0, x + width, y };
-            ::DrawText(hdc, label, label.length(), &rc2,
-                       DT_SINGLELINE | DT_VCENTER);
-        }
-        else // RTL
-        {
-            RECT rc2 = { x, 0, x - width, y };
-            ::DrawText(hdc, label, label.length(), &rc2,
-                       DT_SINGLELINE | DT_VCENTER | DT_RTLREADING);
-        }
+        RECT rc2 = { x, 0, x + width, y };
+        ::DrawText(hdc, label.t_str(), label.length(), &rc2,
+                   drawTextFlags);
     }
+#endif // wxUSE_UXTHEME
 }
 
 void wxStaticBox::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
     RECT rc;
     ::GetClientRect(GetHwnd(), &rc);
+    wxPaintDC dc(this);
 
     // draw the entire box in a memory DC
-    wxMemoryDC memdc;
+    wxMemoryDC memdc(&dc);
     wxBitmap bitmap(rc.right, rc.bottom);
     memdc.SelectObject(bitmap);
 
@@ -531,7 +528,6 @@ void wxStaticBox::OnPaint(wxPaintEvent& WXUNUSED(event))
     // note that it seems to be faster to do 4 small blits here and then paint
     // directly into wxPaintDC than painting background in wxMemoryDC and then
     // blitting everything at once to wxPaintDC, this is why we do it like this
-    wxPaintDC dc(this);
     int borderTop, border;
     GetBordersForSizer(&borderTop, &border);
 
@@ -559,7 +555,8 @@ void wxStaticBox::OnPaint(wxPaintEvent& WXUNUSED(event))
 
     // and also the box itself
     MSWGetRegionWithoutSelf((WXHRGN) hrgn, rc.right, rc.bottom);
-    HDCClipper clipToBg(GetHdcOf(dc), hrgn);
+    wxMSWDCImpl *impl = (wxMSWDCImpl*) dc.GetImpl();
+    HDCClipper clipToBg(GetHdcOf(*impl), hrgn);
 
     // paint the inside of the box (excluding box itself and child controls)
     PaintBackground(dc, rc);
