@@ -26,6 +26,8 @@
 #include "wx/msw/missing.h"
 #include "wx/filesys.h"
 #include "wx/dynlib.h"
+#include "wx/scopeguard.h"
+
 #include <initguid.h>
 #include <wininet.h>
 
@@ -141,18 +143,89 @@ void wxWebViewIE::LoadURL(const wxString& url)
     m_ie.CallMethod("Navigate", wxConvertStringToOle(url));
 }
 
-void wxWebViewIE::DoSetPage(const wxString& html, const wxString& baseUrl)
+namespace
 {
-    BSTR bstr = SysAllocString(OLESTR(""));
-    SAFEARRAY *psaStrings = SafeArrayCreateVector(VT_VARIANT, 0, 1);
-    if (psaStrings != NULL)
+
+// Simple RAII wrapper for accessing SAFEARRAY<VARIANT>. This class is not specific
+// to wxWebView at all and could be extended to work for any type and extracted
+// into a header later if it is ever needed elsewhere, but for now keep it here
+// as it is only used in MakeOneElementVariantSafeArray() which, itself, is
+// only used by DoSetPage() below.
+class wxSafeArrayVariantAccessor
+{
+public:
+    explicit wxSafeArrayVariantAccessor(SAFEARRAY* sa)
+        : m_sa(sa),
+          m_data(DoAccessData(sa))
+    {
+    }
+
+    VARIANT* GetData() const
+    {
+        return m_data;
+    }
+
+    ~wxSafeArrayVariantAccessor()
+    {
+        if ( m_data )
+            SafeArrayUnaccessData(m_sa);
+    }
+
+private:
+    static VARIANT* DoAccessData(SAFEARRAY* sa)
     {
         VARIANT *param;
-        HRESULT hr = SafeArrayAccessData(psaStrings, (LPVOID*)&param);
-        param->vt = VT_BSTR;
-        param->bstrVal = bstr;
+        HRESULT hr = SafeArrayAccessData(sa, (LPVOID*)&param);
+        if ( FAILED(hr) )
+        {
+            wxLogLastError(wxT("SafeArrayAccessData"));
+            return NULL;
+        }
 
-        hr = SafeArrayUnaccessData(psaStrings);
+        return param;
+    }
+
+    SAFEARRAY* const m_sa;
+    VARIANT* const m_data;
+
+    wxDECLARE_NO_COPY_CLASS(wxSafeArrayVariantAccessor);
+};
+
+// Helper function: wrap the given string in a SAFEARRAY<VARIANT> of size 1.
+SAFEARRAY* MakeOneElementVariantSafeArray(const wxString& str)
+{
+    SAFEARRAY* const sa = SafeArrayCreateVector(VT_VARIANT, 0, 1);
+    if ( !sa )
+    {
+        wxLogLastError(wxT("SafeArrayCreateVector"));
+        return NULL;
+    }
+
+    wxSafeArrayVariantAccessor access(sa);
+
+    VARIANT* const param = access.GetData();
+    if ( !param )
+    {
+        SafeArrayDestroy(sa);
+        return NULL;
+    }
+
+    param->vt = VT_BSTR;
+    param->bstrVal = SysAllocString(str.wc_str());
+
+    return sa;
+}
+
+} // anonymous namespace
+
+void wxWebViewIE::DoSetPage(const wxString& html, const wxString& baseUrl)
+{
+    {
+        SAFEARRAY* const psaStrings = MakeOneElementVariantSafeArray(wxString());
+        if ( !psaStrings )
+            return;
+
+        wxON_BLOCK_EXIT1(SafeArrayDestroy, psaStrings);
 
         wxCOMPtr<IHTMLDocument2> document(GetDocument());
 
@@ -161,50 +234,34 @@ void wxWebViewIE::DoSetPage(const wxString& html, const wxString& baseUrl)
 
         document->write(psaStrings);
         document->close();
-
-        SafeArrayDestroy(psaStrings);
-
-        bstr = SysAllocString(html.wc_str());
-
-        // Creates a new one-dimensional array
-        psaStrings = SafeArrayCreateVector(VT_VARIANT, 0, 1);
-        if (psaStrings != NULL)
-        {
-            hr = SafeArrayAccessData(psaStrings, (LPVOID*)&param);
-            param->vt = VT_BSTR;
-            param->bstrVal = bstr;
-            hr = SafeArrayUnaccessData(psaStrings);
-
-            document = GetDocument();
-
-            if(!document)
-                return;
-
-            document->write(psaStrings);
-
-            // SafeArrayDestroy calls SysFreeString for each BSTR
-            SafeArrayDestroy(psaStrings);
-
-            //We send the events when we are done to mimic webkit
-            //Navigated event
-            wxWebViewEvent event(wxEVT_WEBVIEW_NAVIGATED,
-                                 GetId(), baseUrl, "");
-            event.SetEventObject(this);
-            HandleWindowEvent(event);
-
-            //Document complete event
-            event.SetEventType(wxEVT_WEBVIEW_LOADED);
-            event.SetEventObject(this);
-            HandleWindowEvent(event);
-        }
-        else
-        {
-            wxLogError("wxWebViewIE::SetPage() : psaStrings is NULL");
-        }
     }
-    else
+
     {
-        wxLogError("wxWebViewIE::SetPage() : psaStrings is NULL during clear");
+        SAFEARRAY* const psaStrings = MakeOneElementVariantSafeArray(html);
+
+        if ( !psaStrings )
+            return;
+
+        wxON_BLOCK_EXIT1(SafeArrayDestroy, psaStrings);
+
+        wxCOMPtr<IHTMLDocument2> document(GetDocument());
+
+        if(!document)
+            return;
+
+        document->write(psaStrings);
+
+        //We send the events when we are done to mimic webkit
+        //Navigated event
+        wxWebViewEvent event(wxEVT_WEBVIEW_NAVIGATED,
+                             GetId(), baseUrl, "");
+        event.SetEventObject(this);
+        HandleWindowEvent(event);
+
+        //Document complete event
+        event.SetEventType(wxEVT_WEBVIEW_LOADED);
+        event.SetEventObject(this);
+        HandleWindowEvent(event);
     }
 }
 
@@ -223,9 +280,12 @@ wxString wxWebViewIE::GetPageSource() const
             hr = bodyTag->get_parentElement(&htmlTag);
             if(SUCCEEDED(hr))
             {
-                BSTR bstr;
-                htmlTag->get_outerHTML(&bstr);
-                source = wxString(bstr);
+                BSTR bstr = NULL;
+                if ( htmlTag->get_outerHTML(&bstr) == S_OK )
+                {
+                    source = wxString(bstr);
+                    SysFreeString(bstr);
+                }
             }
         }
         return source;
@@ -565,16 +625,18 @@ wxString wxWebViewIE::GetCurrentTitle() const
 {
     wxCOMPtr<IHTMLDocument2> document(GetDocument());
 
+    wxString s;
     if(document)
     {
-        BSTR title;
-        document->get_nameProp(&title);
-        return wxString(title);
-    }
-    else
-    {
-        return "";
+        BSTR title = NULL;
+        if ( document->get_nameProp(&title) == S_OK )
+        {
+            s = title;
+            SysFreeString(title);
+        }
     }
+
+    return s;
 }
 
 bool wxWebViewIE::CanCut() const
@@ -685,9 +747,9 @@ void wxWebViewIE::SetEditable(bool enable)
     if(document)
     {
         if( enable )
-            document->put_designMode(SysAllocString(L"On"));
+            document->put_designMode(wxBasicString("On"));
         else
-            document->put_designMode(SysAllocString(L"Off"));
+            document->put_designMode(wxBasicString("Off"));
 
     }
 }
@@ -698,17 +760,17 @@ bool wxWebViewIE::IsEditable() const
 
     if(document)
     {
-        BSTR mode;
-        document->get_designMode(&mode);
-        if(wxString(mode) == "On")
-            return true;
-        else
-            return false;
-    }
-    else
-    {
-        return false;
+        BSTR mode = NULL;
+        if ( document->get_designMode(&mode) == S_OK )
+        {
+            wxString strMode(mode);
+
+            SysFreeString(mode);
+            if ( strMode == "On" )
+                return true;
+        }
     }
+    return false;
 }
 
 void wxWebViewIE::SelectAll()
@@ -727,9 +789,12 @@ bool wxWebViewIE::HasSelection() const
         HRESULT hr = document->get_selection(&selection);
         if(SUCCEEDED(hr))
         {
-            BSTR type;
-            selection->get_type(&type);
-            sel = wxString(type);
+            BSTR type = NULL;
+            if ( selection->get_type(&type) == S_OK )
+            {
+                sel = wxString(type);
+                SysFreeString(type);
+            }
         }
         return sel != "None";
     }
@@ -763,9 +828,12 @@ wxString wxWebViewIE::GetSelectedText() const
                 hr = disrange->QueryInterface(IID_IHTMLTxtRange, (void**)&range);
                 if(SUCCEEDED(hr))
                 {
-                    BSTR text;
-                    range->get_text(&text);
-                    selected = wxString(text);
+                    BSTR text = NULL;
+                    if ( range->get_text(&text) == S_OK )
+                    {
+                        selected = wxString(text);
+                        SysFreeString(text);
+                    }
                 }
             }
         }
@@ -797,8 +865,11 @@ wxString wxWebViewIE::GetSelectedSource() const
                 if(SUCCEEDED(hr))
                 {
                     BSTR text;
-                    range->get_htmlText(&text);
-                    selected = wxString(text);
+                    if ( range->get_htmlText(&text) == S_OK )
+                    {
+                        selected = wxString(text);
+                        SysFreeString(text);
+                    }
                 }
             }
         }
@@ -837,9 +908,12 @@ wxString wxWebViewIE::GetPageText() const
         HRESULT hr = document->get_body(&body);
         if(SUCCEEDED(hr))
         {
-            BSTR out;
-            body->get_innerText(&out);
-            text = wxString(out);
+            BSTR out = NULL;
+            if ( body->get_innerText(&out) == S_OK )
+            {
+                text = wxString(out);
+                SysFreeString(out);
+            }
         }
         return text;
     }
@@ -863,8 +937,8 @@ void wxWebViewIE::RunScript(const wxString& javascript)
             VARIANT level;
             VariantInit(&level);
             V_VT(&level) = VT_EMPTY;
-            window->execScript(SysAllocString(javascript.wc_str()),
-                               SysAllocString(language.wc_str()),
+            window->execScript(wxBasicString(javascript),
+                               wxBasicString(language),
                                &level);
         }
     }
@@ -909,7 +983,7 @@ bool wxWebViewIE::CanExecCommand(wxString command) const
     {
         VARIANT_BOOL enabled;
 
-        document->queryCommandEnabled(SysAllocString(command.wc_str()), &enabled);
+        document->queryCommandEnabled(wxBasicString(command), &enabled);
 
         return (enabled == VARIANT_TRUE);
     }
@@ -926,7 +1000,7 @@ void wxWebViewIE::ExecCommand(wxString command)
 
     if(document)
     {
-        document->execCommand(SysAllocString(command.wc_str()), VARIANT_FALSE, VARIANT(), NULL);
+        document->execCommand(wxBasicString(command), VARIANT_FALSE, VARIANT(), NULL);
     }
 }
 
@@ -945,8 +1019,7 @@ wxCOMPtr<IHTMLDocument2> wxWebViewIE::GetDocument() const
 
 bool wxWebViewIE::IsElementVisible(wxCOMPtr<IHTMLElement> elm)
 {
-    wxCOMPtr<IHTMLElement> elm1 = elm;
-    BSTR tmp_bstr;
+    wxCOMPtr<IHTMLElement> elm1 = elm;    
     bool is_visible = true;
     //This method is not perfect but it does discover most of the hidden elements.
     //so if a better solution is found, then please do improve.
@@ -956,6 +1029,8 @@ bool wxWebViewIE::IsElementVisible(wxCOMPtr<IHTMLElement> elm)
         if(SUCCEEDED(elm1->QueryInterface(wxIID_IHTMLElement2, (void**) &elm2)))
         {
             wxCOMPtr<wxIHTMLCurrentStyle> style;
+            BSTR tmp_bstr = NULL;
+
             if(SUCCEEDED(elm2->get_currentStyle(&style)))
             {
                 //Check if the object has the style display:none.
@@ -964,12 +1039,15 @@ bool wxWebViewIE::IsElementVisible(wxCOMPtr<IHTMLElement> elm)
                 {
                     is_visible = false;
                 }
+                SysFreeString(tmp_bstr);
+                tmp_bstr = NULL;
                 //Check if the object has the style visibility:hidden.
                 if((is_visible && (style->get_visibility(&tmp_bstr) != S_OK)) ||
                   (tmp_bstr != NULL && wxCRT_StricmpW(tmp_bstr, L"hidden") == 0))
                 {
                     is_visible = false;
                 }
+                SysFreeString(tmp_bstr);                
                 style->Release();
             }
             elm2->Release();
