@@ -29,10 +29,6 @@
 
 #include "wx/osx/private.h"
 
-#if wxOSX_USE_COCOA_OR_CARBON
-    #include <QuickTime/QuickTime.h>
-#endif
-
 // ----------------------------------------------------------------------------
 // wxDataFormat
 // ----------------------------------------------------------------------------
@@ -463,7 +459,7 @@ bool wxDataObject::GetFromPasteboard( void * pb )
                             // indicate the expected format for the type, benefiting from native conversions eg utf8 -> utf16
                             flavorType = (CFStringRef) wxDataFormat( flavorFormat.GetType()).GetFormatId();
                         }
-                        
+
                         err = PasteboardCopyItemFlavorData( pasteboard, itemID, flavorType , &flavorData );
                         if ( err == noErr )
                         {
@@ -472,14 +468,20 @@ bool wxDataObject::GetFromPasteboard( void * pb )
                             {
                                  // revert the translation and decomposition to arrive at a proper utf8 string again
                                 CFURLRef url = CFURLCreateWithBytes( kCFAllocatorDefault, CFDataGetBytePtr( flavorData ), flavorDataSize, kCFStringEncodingUTF8, NULL );
-                                CFStringRef cfString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
-                                CFRelease( url );
-                                CFMutableStringRef cfMutableString = CFStringCreateMutableCopy(NULL, 0, cfString);
-                                CFRelease( cfString );
-                                CFStringNormalize(cfMutableString,kCFStringNormalizationFormC);
-                                wxString path = wxCFStringRef(cfMutableString).AsString();
-                                if (!path.empty())
-                                    filenamesPassed += path + wxT("\n");
+                                if ( url )
+                                {
+                                    CFStringRef cfString = CFURLCopyFileSystemPath( url, kCFURLPOSIXPathStyle );
+                                    CFRelease( url );
+                                    if ( cfString )
+                                    {
+                                        CFMutableStringRef cfMutableString = CFStringCreateMutableCopy(NULL, 0, cfString);
+                                        CFRelease( cfString );
+                                        CFStringNormalize(cfMutableString,kCFStringNormalizationFormC);
+                                        wxString path = wxCFStringRef(cfMutableString).AsString();
+                                        if (!path.empty())
+                                            filenamesPassed += path + wxT("\n");
+                                    }
+                                }
                             }
                             else
                             {
