@@ -224,7 +224,7 @@ private:
         CPPUNIT_TEST( TestTimeParse );
         CPPUNIT_TEST( TestTimeSpanFormat );
         CPPUNIT_TEST( TestTimeTicks );
-        CPPUNIT_TEST( TestParceRFC822 );
+        CPPUNIT_TEST( TestParseRFC822 );
         CPPUNIT_TEST( TestDateParse );
         CPPUNIT_TEST( TestDateParseISO );
         CPPUNIT_TEST( TestDateTimeParse );
@@ -232,6 +232,7 @@ private:
         CPPUNIT_TEST( TestTimeArithmetics );
         CPPUNIT_TEST( TestDSTBug );
         CPPUNIT_TEST( TestDateOnly );
+        CPPUNIT_TEST( TestConvToFromLocalTZ );
     CPPUNIT_TEST_SUITE_END();
 
     void TestLeapYears();
@@ -244,7 +245,7 @@ private:
     void TestTimeParse();
     void TestTimeSpanFormat();
     void TestTimeTicks();
-    void TestParceRFC822();
+    void TestParseRFC822();
     void TestDateParse();
     void TestDateParseISO();
     void TestDateTimeParse();
@@ -252,6 +253,7 @@ private:
     void TestTimeArithmetics();
     void TestDSTBug();
     void TestDateOnly();
+    void TestConvToFromLocalTZ();
 
     DECLARE_NO_COPY_CLASS(DateTimeTestCase)
 };
@@ -816,6 +818,8 @@ void DateTimeTestCase::TestTimeFormat()
         }
     }
 
+    CPPUNIT_ASSERT(wxDateTime::Now().Format("%") == "%");
+
     wxDateTime dt;
 
 #if 0
@@ -948,7 +952,7 @@ void DateTimeTestCase::TestTimeTicks()
 }
 
 // test parsing dates in RFC822 format
-void DateTimeTestCase::TestParceRFC822()
+void DateTimeTestCase::TestParseRFC822()
 {
     static const struct ParseTestData
     {
@@ -1447,4 +1451,23 @@ void DateTimeTestCase::TestDateOnly()
     CPPUNIT_ASSERT_EQUAL( wxDateTime::Today(), wxDateTime::Now().GetDateOnly() );
 }
 
+void DateTimeTestCase::TestConvToFromLocalTZ()
+{
+    // Choose a date when the DST is on in many time zones: in this case,
+    // converting to/from local TZ does modify the object because it
+    // adds/subtracts DST to/from it, so to get the expected results we need to
+    // explicitly disable DST support in these functions.
+    wxDateTime dt(18, wxDateTime::Apr, 2017, 19);
+
+    CPPUNIT_ASSERT_EQUAL( dt.FromTimezone(wxDateTime::Local, true), dt );
+    CPPUNIT_ASSERT_EQUAL( dt.ToTimezone(wxDateTime::Local, true), dt );
+
+    // And another one when it is off: in this case, there is no need to pass
+    // "true" as "noDST" argument to these functions.
+    dt = wxDateTime(18, wxDateTime::Jan, 2018, 19);
+
+    CPPUNIT_ASSERT_EQUAL( dt.FromTimezone(wxDateTime::Local), dt );
+    CPPUNIT_ASSERT_EQUAL( dt.ToTimezone(wxDateTime::Local), dt );
+}
+
 #endif // wxUSE_DATETIME
